package datastructures.binarytree;

public class BinaryTreeWithParents implements Tree {
    private TreeNodeWithParent root;

    public BinaryTreeWithParents(int key, String value) {
        root = new TreeNodeWithParent(key, value, null);
    }

    public void add(int key, String value) {
        if (key != 0) { //check if key is valid

            TreeNodeWithParent node = getForAdding(key, root);
            if (key == node.getKey()) {
                node.setValue(node.getValue() + value);
                return;
            }
            if (key < node.getKey()) {
                node.setLeft(new TreeNodeWithParent(key, value, node));
            } else {
                node.setRight(new TreeNodeWithParent(key, value, node));
            }
        }
    }

    public void traverseTree() {
        traverseTree(root);
    }

    @Override
    public boolean contains(String value) {
        return contains(value, root);
    }

    private boolean contains(String s, TreeNodeWithParent node) {
        boolean res;
        if (node != null) {
            res = contains(s, node.getLeft());
            res = res || contains(s, node.getRight());
            return res || node.getValue().equals(s);
        }
        return false;
    }

    private void traverseTree(TreeNodeWithParent node) {
        if (node != null) {
            traverseTree(node.getLeft());
            node.getValue();//do something with value
            traverseTree(node.getRight());
        }
    }

    public String get(int key) {
        TreeNodeWithParent node = get(key,root);
        return node == null ? null : node.getValue();
    }

    private TreeNodeWithParent get(int key, TreeNodeWithParent node) {
        if (node == null)//just checking if a null-instance was passed
            return null;

        if (key == node.getKey())//element found
            return node;

        //search in left or right childtree
        if (key < node.getKey()) {
            return get(key, node.getLeft());
        } else {
            return get(key, node.getRight());
        }
    }

    private TreeNodeWithParent getForAdding(int key, TreeNodeWithParent node) {
        if (node == null)//just checking if a null-instance was passed
            return null;

        if (key == node.getKey())//element found
            return node;

        //search in left or right childtree
        if (key < node.getKey()) {
            if (node.getLeft() == null)//if this node.left is a leaf, return this node
                return node;
            return getForAdding(key, node.getLeft());
        } else {
            if (node.getRight() == null)//if this node.right is a leaf, return this node
                return node;
            return getForAdding(key, node.getRight());
        }
    }

    public String toString() {
        return toString(root);
    }

    private String toString(TreeNodeWithParent node) {
        if (root != null) {
            String res = "";
            if (node.getLeft() != null)
                res += " " + toString(node.getLeft());
            res += " " + node.getValue();
            if (node.getRight() != null)
                res += " " + toString(node.getRight());
            return res;
        }
        return "";
    }
}

class TreeNodeWithParent {
    private int key;
    private String value;
    private TreeNodeWithParent left, right, parent;

    public TreeNodeWithParent(int key, String value, TreeNodeWithParent parent) {
        this.key = key;
        this.value = value;
        this.parent = parent;
    }

    public int getKey() {
        return key;
    }

    public void setKey(int key) {
        this.key = key;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public TreeNodeWithParent getLeft() {
        return left;
    }

    public void setLeft(TreeNodeWithParent left) {
        this.left = left;
    }

    public TreeNodeWithParent getRight() {
        return right;
    }

    public void setRight(TreeNodeWithParent right) {
        this.right = right;
    }
}