package datastructures.list;

public class LinearList implements List {
    private LinearListElement first, last;

    public LinearList(String s) {
        first = last = new LinearListElement(s);
    }

    public void add(String s) {
        last.setNext(new LinearListElement(s));//set the new element as last.next
        last = last.getNext(); //set this.last to the new element
    }

    public String get(int i) {
        return get(i, first);
    }

    private String get(int i, LinearListElement el) {
        if(el == null)//end of list
            return null;
        if(i == 0)//index reached
            return el.getValue();
        return get(i-1,el.getNext());//index not reached.. go further
    }

    public boolean contains(String s) {
        return contains(s, first);
    }

    private boolean contains(String s, LinearListElement el) {
        if (el == null)//reached end of list
            return false;
        if (s.equals(el.getValue())) //value found
            return true;
        return contains(s, el.getNext());//go further
    }

    public boolean remove(int i) {
        return remove(i, first);
    }

    private boolean remove(int i, LinearListElement el) {
        if (el == null || (i == 1 && el.getNext() == null))//(i == 1 && el.getNext() == null) to check if index out of bounce
            return false;
        if (i == 1) {
            el.setNext(el.getNext().getNext());
            return true;
        }
        return remove(i - 1, el.getNext());
    }

    public String removeFirst() {
        String res = first.getValue();
        first = first.getNext();
        return res;
    }

    public String toString(){
        return toString(first);
    }

    private String toString(LinearListElement el) {
        if(el != null){
            return el.getValue() + toString(el.getNext());
        }
        return "";
    }

    @Override
    public boolean isEmpty(){
        return first == null;
    }
}

class LinearListElement {
    private String value;
    private LinearListElement next;

    public LinearListElement(String value) {
        this.value = value;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public LinearListElement getNext() {
        return next;
    }

    public void setNext(LinearListElement next) {
        this.next = next;
    }
}
