package other.treeiterator;

import datastructures.binarytree.Tree;

import java.util.Iterator;

public class BinaryTree implements Tree, Iterable<String> {
    private TreeNode root;

    public BinaryTree(int key, String value) {
        root = new TreeNode(key, value);
    }

    public void add(int key, String value) {
        if (key != 0) { //check if key is valid
            TreeNode node = getParentForKey(key, root);
            if (key == node.getKey()) {
                node.setValue(node.getValue() + value);
                return;
            }
            if (key < node.getKey()) {
                node.setLeft(new TreeNode(key, value));
            } else {
                node.setRight(new TreeNode(key, value));
            }
        }
    }

    public void traverseTree() {
        traverseTree(root);
    }

    private void traverseTree(TreeNode node) {
        if (node != null) {
            traverseTree(node.getLeft());
            node.getValue();//do something with value
            traverseTree(node.getRight());
        }
    }

    @Override
    public boolean contains(String value) {
        return contains(value, root);
    }

    private boolean contains(String s, TreeNode node) {
        boolean res;
        if (node != null) {
            res = contains(s, node.getLeft());
            res = res || contains(s, node.getRight());
            return res || node.getValue().equals(s);
        }
        return false;
    }

    public String get(int key) {
        TreeNode node = get(key, root);
        return node == null ? null : node.getValue();
    }

    private TreeNode get(int key, TreeNode node) {
        if (node == null)//just checking if a null-instance was passed
            return null;

        if (key == node.getKey())//element found
            return node;

        //search in left or right childtree
        if (key < node.getKey()) {
            return get(key, node.getLeft());
        } else {
            return get(key, node.getRight());
        }
    }

    private TreeNode getParentForKey(int key, TreeNode node) {
        if (node == null)//checking if a null-instance was passed
            return null;

        if (key == node.getKey())//element found
            return node;

        //search in left or right childtree
        if (key < node.getKey()) {
            if (node.getLeft() == null)//if this node.left is a leaf, return this node
                return node;
            return getParentForKey(key, node.getLeft());
        } else {
            if (node.getRight() == null)//if this node.right is a leaf, return this node
                return node;
            return getParentForKey(key, node.getRight());
        }
    }

    public String toString() {
        return toString(root);
    }

    private String toString(TreeNode node) {
        if (root != null) {
            String res = "";
            if (node.getLeft() != null) {
                res += " " + toString(node.getLeft());
            }
            res += " " + node.getValue();
            if (node.getRight() != null)
                res += " " + toString(node.getRight());
            return res;
        }
        return "";
    }


    @Override
    public Iterator<String> iterator() {
        return new TreeIterator(root);
    }
}

class TreeNode {
    private int key;
    private String value;
    private TreeNode left, right;

    public TreeNode(int key, String value) {
        this.key = key;
        this.value = value;
    }

    public int getKey() {
        return key;
    }

    public void setKey(int key) {
        this.key = key;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public TreeNode getLeft() {
        return left;
    }

    public void setLeft(TreeNode left) {
        this.left = left;
    }

    public TreeNode getRight() {
        return right;
    }

    public void setRight(TreeNode right) {
        this.right = right;
    }
}

