package probetests.probetest3;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

// 'Constant' implements 'Polynomial' and represents a polynomial with degree 0 corresponding
// to a constant value (regardless of the 'x' used for evaluation).
// 'Constant' is used by class 'HornerScheme'.
// The iterator of this class iterates over only one value and therefore behaves as follows:
// The 'next' method returns the constant value, if it is called for the first time.
// 'hasNext' is 'true' only if 'next' has not been called.
//
public class Constant implements Polynomial {

    private int constant;

    // Initializes this object.
    // Precondition: c != 0.
    public Constant(int c) {
        this.constant = c;
    }

    @Override
    public int degree() {
        return 0;
    }

    @Override
    public List<Integer> coefficients() {
        List<Integer> list = new ArrayList<>();
        list.add(constant);
        return list;
    }

    @Override
    public double eval(double x) {
        return (double) constant;
    }

    @Override
    public Iterator<Integer> iterator() {
        return new Iterator<Integer>() {
            boolean hasNext = true;

            @Override
            public boolean hasNext() {
                return hasNext;
            }

            @Override
            public Integer next() {
                if (!hasNext) {
                    throw new NoSuchElementException();
                }
                hasNext = false;
                return constant;
            }
        };
    }

    @Override
    public int hashCode() {
        return 31 * constant;
    }

    @Override
    public boolean equals(Object obj) {
        if(obj == null || obj.getClass() != this.getClass())
            return false;
        if(obj == this)
            return true;
        Constant con = (Constant) obj;
        return con.constant == constant;
    }

    @Override
    public String toString() {
        return "" + constant;
    }
}


