package probetests.probetest4;

import java.util.*;


// This class implements 'StringOperator' and represents a composition of two 'StringOperator' objects that
// are applied in succession (both can be 'ComposedOperator' objects themselves).
// Calling this object's 'apply' method returns the result of applying the second 'StringOperator'
// on the result of applying the first one on the specified string.
// 'ComposedOperator' objects can be build using the 'andThen' method specified in the interface 'StringOperator'.
//
public class ComposedOperator implements StringOperator {
    private StringOperator first;
    private StringOperator second;


    public ComposedOperator(StringOperator first, StringOperator second) {
        this.first = first;
//        this.second = new ComposedOperator(second);
        this.second = second;
    }

    @Override
    public String apply(String operand) {
        String res = first.apply(operand);
        if (second != null)
            res = second.apply(res);
        return res;
    }

    @Override
    public StringOperator andThen(StringOperator after) {
        //My first attempt... did not work but maybe interesting why
//        if(second == null)
//            second = after;
//        else if(second.getClass() == ReplaceOperator.class)
//            second = second.andThen(after);
//        else
//            second.andThen(after);
//        return this;
        return new ComposedOperator(this, after);
    }

    @Override
    public Iterator<StringOperator> iterator() {
        return new Iterator<StringOperator>() {
            private Iterator<StringOperator> iterSecond = second.iterator();
            private Iterator<StringOperator> iterFirst = first.iterator();

            @Override
            public boolean hasNext() {
                return iterSecond.hasNext() || iterSecond.hasNext();
            }

            @Override
            public StringOperator next() {
                if (iterFirst.hasNext()) {
                    return iterFirst.next();
                } else {
                    return iterSecond.next();
                }
            }
        };
    }

    @Override
    public int hashCode() {
        return first.hashCode() + (second != null ? second.hashCode() : 0);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null || obj.getClass() != getClass())
            return false;
        if (obj == this)
            return true;
        ComposedOperator cp = (ComposedOperator) obj;
        return first.equals(cp.first) && Objects.equals(cp.second, second);
    }

    @Override
    public String toString() {
        String res = "";
        res += first.toString();
        if (second != null)
            res += "\n" + second.toString();
        return res;
    }
}

