package probetests.probetest4;

import java.util.*;

// This class implements 'StringOperator' and represents a simple replacement operation on a string.
// It replaces the all occurrences of the specified 'oldChar' with 'newChar'.
// The iterator of this class iterates over only one element and therefore behaves as follows:
// The 'next' method returns 'this', if it is called for the first time.
// 'hasNext' is 'true' only if 'next' has not been called.
//
public class ReplaceOperator implements StringOperator {

    private char oldChar;
    private char newChar;

    // Constructor of the 'ReplaceOperator'.
    public ReplaceOperator(char oldChar, char newChar) {
        this.newChar = newChar;
        this.oldChar = oldChar;
    }

    @Override
    public String apply(String operand) {
        char[] operandArray = operand.toCharArray();
        for (int i = 0; i < operandArray.length; i++) {
            if (operandArray[i] == oldChar)
                operandArray[i] = newChar;
        }

        return String.valueOf(operandArray);
    }

    @Override
    public StringOperator andThen(StringOperator after) {
        ComposedOperator co = new ComposedOperator(this, after);
        return co;
    }

    @Override
    public Iterator<StringOperator> iterator() {
        return new Iterator<StringOperator>() {
            boolean hasNext = true;

            @Override
            public boolean hasNext() {
                return hasNext;
            }

            @Override
            public StringOperator next() {
                if (hasNext) {
                    hasNext = false;
                    return ReplaceOperator.this;
                } else {
                    throw new NoSuchElementException("No operator");
                }
            }
        };
    }

    @Override
    public int hashCode() {
        return Objects.hash(oldChar, newChar);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null || obj.getClass() != getClass())
            return false;
        if (obj == this)
            return true;
        ReplaceOperator ro = (ReplaceOperator) obj;
        return ro.newChar == this.newChar && ro.oldChar == this.oldChar;
    }

    @Override
    public String toString() {
        return "replace '" + oldChar + "' with '" + newChar + "'";
    }
}

