package vowi.Test1_19S.probetest;

// This class can be modified and is for testing your solution.
// Modifications will NOT be reviewed or assessed.
public class PraxisApplication {
    public static void main(String[] args) {
        Monom m = new Monom(3, 2);
        System.out.println(m); // 3*x^2
        System.out.println(m.eval(-5)); // 75
        System.out.println(m.eval(0)); // 0

        System.out.println(m.combine(new Monom(-1, 2))); // true
        System.out.println(m); // 2*x^2

        System.out.println(m.combine(new Monom(-1, 3))); // false
        System.out.println(m); // 2*x^2

        Polynom p1 = new Polynom(new int[]{2, 0, 3, 1});
        Polynom p2 = new Polynom(new int[]{0, 0, 3, -1});

        System.out.println(p1); // 2*x^0 + 3*x^2 + 1*x^3
        System.out.println(p1.eval(-5)); // -48
        System.out.println(p2); // 3*x^2 + -1*x^3

        p1.add(p2);

        System.out.println(p1); // 2*x^0 + 6*x^2 + 0*x^3

        p1 = new Polynom(new int[]{2, 0, 3, 1});
        p2 = new Polynom(new int[]{0, 0, 3, 1});

        p2.add(p1);

        System.out.println(p2); // 2*x^0 + 6*x^2 + 2*x^3
    }
}

// This class represents monomials of the form a*x^d ("a mal x hoch d"), where
// 'a' is the coefficient of the monomial and 'd' is the degree.
class Monom {
    private int coeff;
    private int degree;

    // A constructor with the coefficient 'coeff' and the 'degree' of this monomial.
    public Monom(int coeff, int degree) {
        this.coeff = coeff;
        this.degree = degree;
    }

    // Copy-constructor: uses 'coeff' and 'degree' of 'm' to initialize this monomial.
    public Monom(Monom m) {
        this.degree = m.degree;
        this.coeff = m.coeff;
    }

    // Adds the monomial 'm' to this monomial, if both monomials have the same degree.
    // In this case this monomial's coefficient is replaced by the sum of this monomial's
    // coefficient and the coefficient of 'm'. In this case the method returns 'true'.
    // If 'm' has not the same degree as 'this', the method has no effect and
    // returns 'false'.
    public boolean combine(Monom m) {
        if (m.degree == this.degree) {
            this.coeff += m.coeff;
            return true;
        }
        return false;
    }

    // Returns 'true' if 'm' has a higher degree than 'this', and 'false' otherwise.
    public boolean lowerDegreeThan(Monom m) {
        return m.degree > this.degree;
    }

    // Returns the value of the monomial for a specified value of 'x'
    public int eval(int x) {
        return this.coeff * (int) Math.pow(x, degree);
    }

    // Returns a representation in mathematical notation, e.g. of the form "5*x^2".
    public String toString() {
        return coeff + "*x^" + degree;
    }
}
// This class represents a polynomial. A polynomial is a sum of multiple monomials (class 'Monom'),
// such as 3*x^1 + -1*x^2 + 1*x^5.
// 'Polynom' uses a binary search tree to store its monomials. The degree of the monomial is the key.
// A specific degree exists at most once in the polynomial.
// TODO: define further classes for the implementation of the binary search tree, if needed

class Polynom {
    private TreeElement root;

    // Initializes this polynomial with multiple monomials. The coefficients of the monomials are
    // specified by 'coeffs', where coeffs[i] is the coefficient of the monomial of degree i.
    // Entries with value 0 are ignored, i.e. corresponding monomials are not stored in the polynomial.
    public Polynom(int[] coeffs) {
        for (int i = 0; i < coeffs.length; i++) {
            if (coeffs[i] != 0) {
                if (root == null) {
                    root = new TreeElement(i, new Monom(coeffs[i], i));
                } else {
                    add(coeffs[i], i, root);
                }
            }
        }
    }

    private void add(int coeff, int key, TreeElement node) {
        TreeElement el = findParentForKey(node, key);
        Monom m = new Monom(coeff, key);

        if (el.getKey() == key)
            el.getValue().combine(m);
        else if (key < el.getKey())
            el.setLeft(new TreeElement(key, m));
        else
            el.setRight(new TreeElement(key, m));
    }

    private TreeElement findParentForKey(TreeElement node, int key) {
        if (node.getKey() == key)
            return node;
        if (key < node.getKey()) {
            if (node.getLeft() == null)
                return node;
            return findParentForKey(node.getLeft(), key);
        } else {
            if (node.getRight() == null)
                return node;
            return findParentForKey(node.getRight(), key);
        }
    }

    // Adds the monomial specified by 'coeff' and 'degree' to this polynomial, if coeff != 0,
    // otherwise 'add' has no effect.
    // If this polynomial already has a monomial of the same degree, no new monomial is added, instead
    // the coefficient of the monomial is modified accordingly ('combine' is called).
    public void add(int coeff, int degree) {
        add(coeff, degree, root);
    }

    // Adds all monomials of 'p' to this polynomial.
    // (The rules of 'add(int,int)' apply for each monomial to be added.)
    public void add(Polynom p) {
        addP(p.root);
    }

    private void addP(TreeElement node) {
        if (node == null) return;
        add(node.getValue().eval(1), node.getKey());
        addP(node.getRight());
        addP(node.getLeft());
    }

    // Returns the value of the polynomial for a specified value of 'x'
    public int eval(int x) {
        return eval(x, root);
    }

    private int eval(int x, TreeElement node) {
        if (node == null)
            return 0;
        int res = node.getValue().eval(x);
        res += eval(x, node.getLeft());
        res += eval(x, node.getRight());
        return res;
    }

    // Returns a polynomial representation in mathematical notation such as
    // "2*x^0 + 6*x^2 + -2*x^3", where monomials are ordered ascending according to
    // their degree. Note that a positive sign of the leftmost coefficient is
    // not shown.
    // Returns the string "0" if the polynomial has no monomials (is empty).
    public String toString() {
        return toString(root).substring(3);
    }

    private String toString(TreeElement node) {
        if (node == null)
            return "";
        String res = " + " + node.getValue().toString();
        res += toString(node.getLeft());
        res += toString(node.getRight());
        return res;
    }
}

class TreeElement {
    private TreeElement left, right;
    private int key;
    private Monom value;

    public TreeElement(int key, Monom value) {
        this.key = key;
        this.value = value;
    }

    public TreeElement getLeft() {
        return left;
    }

    public void setLeft(TreeElement left) {
        this.left = left;
    }

    public TreeElement getRight() {
        return right;
    }

    public void setRight(TreeElement right) {
        this.right = right;
    }

    public int getKey() {
        return key;
    }

    public void setKey(int key) {
        this.key = key;
    }

    public Monom getValue() {
        return value;
    }

    public void setValue(Monom value) {
        this.value = value;
    }
}