package vowi.Test1_19S.rationaltree;

// This class can be modified and is for testing your solution.
// Modifications will NOT be reviewed or assessed.
//
public class PraxisApplication {

    public static void main(String[] args) {

        Rational r = Rational.create(1, 2);
        System.out.println(r); // 1/2
        System.out.println(r.value()); // 0.5

        Rational s = Rational.create(2, 0);
        System.out.println(s); // NaN
        System.out.println(s.value()); // NaN

        System.out.println(s.mult(r)); // NaN
        System.out.println(r.mult(s)); // NaN
        System.out.println(r.mult(r)); // 1/4
        System.out.println(s.mult(s)); // NaN

        RationalCollection m1 = new RationalCollection();

        m1.add(r);
        m1.add(Rational.create(2, -3));
        m1.add(Rational.create(-3, -4));
        m1.add(Rational.create(1, -3));

        System.out.println(m1); // [-2/3, -1/3, 1/2, 3/4]

        System.out.println(m1.product(-1)); // 6/72
        System.out.println(m1.product(0.5)); // 3/8
        System.out.println(m1.product(1)); // 1/1

    }
}

// This class represents a sorted collection of elements of type 'Rational'.
// The elements are sorted with respect to their value.
// 'RationalCollection' uses a binary search tree for storing elements.
// For simplicity it is assumed that rounding errors do not matter; don't worry about them.
// Duplicate elements (entries) can occur in the collection.
// TODO: define further classes for the implementation of the binary search tree, if needed (either in this file
//  or in separate files)
//
class RationalCollection {
    private TreeElement root;

    // Adds the specified Rational 'r' to this set.
    // Preconditions: r != null and r is not NaN (need not be checked).
    public void add(Rational r) {
        if (root == null) {
            root = new TreeElement(r.value(), r);
            return;
        }
        TreeElement el = findParentForAdding(root, r.value());
        if(r.value() <= el.getKey())
            el.setLeft(new TreeElement(r.value(), r));
        else
            el.setRight(new TreeElement(r.value(), r));
    }

    private TreeElement findParentForAdding(TreeElement node, double key) {
        if(key <= node.getKey()){
            if(node.getLeft() == null)
                return node;
            else
                return findParentForAdding(node.getLeft(), key);
        } else {
            if(node.getRight() == null)
                return node;
            else
                return findParentForAdding(node.getRight(), key);
        }
    }

    // Returns the product of all elements of this collection with values
    // greater than or equal to 'lower'.
    // Returns 1/1 if there are no such elements in the collection.
    public Rational product(double lower) {
        Rational res = new Rational(1,1);
        return product(lower, root, res);
    }

    private Rational product(double lower, TreeElement node, Rational res) {
        if(node == null)
            return res;
        if(node.getKey() >= lower)
            res = res.mult(node.getValue());

        if(node.getLeft() != null) {
            res = product(lower, node.getLeft(), res);
        }
        if(node.getRight() != null)
            res = product(lower, node.getRight(), res);
        return res;
    }

    // Returns a string representation of this collection. Elements are in
    // ascending order according to their values. Example: "[-2/3, -1/3, 1/2, 3/4]"
    public String toString() {
        String res = "[";
        String value = toString(root, res);
        value = value.substring(0, value.length()-1);
        value = value.substring(2);
        res += value + "]";
        return res;
    }

    private String toString(TreeElement root, String res) {
        if(root == null)
            return res;
        if(root.getLeft() != null)
            res = toString(root.getLeft(), res);
        res +=  " "+root.getValue().toString() +",";
        if(root.getRight() != null)
            res = toString(root.getRight(), res);
        return res;
    }
}

class TreeElement {
    private TreeElement left, right;
    private double key;
    private Rational value;

    public TreeElement(double key, Rational value) {
        this.key = key;
        this.value = value;
    }

    public TreeElement getLeft() {
        return left;
    }

    public void setLeft(TreeElement left) {
        this.left = left;
    }

    public TreeElement getRight() {
        return right;
    }

    public void setRight(TreeElement right) {
        this.right = right;
    }

    public double getKey() {
        return key;
    }

    public void setKey(double key) {
        this.key = key;
    }

    public Rational getValue() {
        return value;
    }

    public void setValue(Rational value) {
        this.value = value;
    }
}

// Rational represents a fraction of two integers such as
// 1/2 (with numerator 1, denominator 2 and value 0.5)
// or -3/4 (with numerator -3, denominator 4 and value -0.75).
// Rational can also represent "not a number" (NaN).
//
class Rational {
    private int numerator;
    private int denominator;
    private static Rational NaN = new Rational();


    // Returns a new instance of 'Rational' with numerator and denominator.
    // If the denominator is 0, the object returned is an instance of Rational
    // corresponding to NaN (not a number).
    public static Rational create(int numerator, int denominator) {
        if (denominator == 0)
            return NaN;
        return new Rational(numerator, denominator);
    }

    public Rational() {
    }

    public Rational(int numerator, int denominator) {
        this.numerator = numerator;
        this.denominator = denominator;
    }

    // Returns the product of 'this' and 'r'.
    // For example, the product of 2/3 and 4/5 is 8/15.
    // If either 'this' or 'r' is NaN, the method returns NaN.
    public Rational mult(Rational r) {
        if (this == NaN || r == NaN)
            return NaN;
        return new Rational(this.numerator * r.numerator,
                this.denominator * r.denominator);
    }

    // Returns the value of 'this'.
    // If 'this' represents NaN, the method returns Double.NaN.
    public double value() {
        if (this == NaN)
            return Double.NaN;
        return (double) numerator / (double) denominator;
    }

    // Returns a representation with numerator and denominator such as "-1/2".
    // If the value is negative, the sign is the first character, i.e., the denominator
    // in the resulting string is always a positive number
    // (making use of the equivalence between x/-y and -x/y).
    // If 'this' is NaN, the method returns "NaN".
    public String toString() {
        if (this == NaN)
            return "NaN";
        if (denominator < 0)
            return (-1*numerator) + "/" + (-1*denominator);
        return numerator + "/" + denominator;
    }
}