package vowi.Test1_19S.ringliste;

// This class can be modified and is for testing your solution.
// Modifications will NOT be reviewed or assessed.
//
public class PraxisApplication {
    public static void main(String[] args) {

        Fighter fighter1 = new Fighter("The Viper", 9);
        Fighter fighter2 = new Fighter("The Mountain", 10);
        fighter1.fights(fighter2);
        System.out.println(fighter1.isDead()); // true
        System.out.println(fighter2.isDead()); // false

        Fighter biter = new Fighter("Freddie", 2);

        Team undead = new Team(new Fighter[]{biter,
                new Fighter("Creeper", 2),
                new Fighter("Dan", 2),
                new Fighter("Abey", 2)});

        Team living = new Team(new Fighter[]{new Fighter("Andrea", 5),
                new Fighter("Bernd", 4)});

        System.out.println(living + " : " + undead);
        // [ *Andrea(5)* Bernd(4) ] : [ *Freddie(2)* Creeper(2) Dan(2) Abey(2) ]

        while (!(undead.isEmpty() || living.isEmpty())) {
            living.fight(undead);
            System.out.println(living + " : " + undead);
        }
        /*
        [ *Andrea(5)* Bernd(4) ] : [ *Freddie(2)* Creeper(2) Dan(2) Abey(2) ]
        [ Andrea(3) *Bernd(4)* ] : [ *Creeper(2)* Dan(2) Abey(2) ]
        [ *Andrea(3)* Bernd(2) ] : [ *Dan(2)* Abey(2) ]
        [ Andrea(1) *Bernd(2)* ] : [ *Abey(2)* ]
        [ *Andrea(1)* ] : [ ]
         */
    }
}

// This class represents a fighting character in a role playing game. Every
// fighter has a name and a number of hitpoints.
//
class Fighter {
    private String name;
    private int hitpoints;

    // A constructor with the 'name' of the fighter and his 'hitpoints'.
    public Fighter(String name, int hitpoints) {
        this.name = name;
        this.hitpoints = hitpoints;
    }

    // Simulates a fight-to-the-death between 'this' and 'opponent'.
    // The fighter with more 'hitpoints' wins. His hitpoints are reduced
    // by the hitpoints of the losing fighter. The losing fighter is dead
    // after the fight (see method 'isDead').
    // In the case of equal hitpoints, both fighters are dead after the fight.
    // Precondition: opponent != null (needs not be checked).
    public void fights(Fighter opponent) {
        if (opponent.hitpoints <= hitpoints) {
            hitpoints -= opponent.hitpoints;
            opponent.hitpoints = 0;
        } else {
            opponent.hitpoints -= hitpoints;
            hitpoints = 0;
        }
    }

    // Returns 'false' in the case of positive hitpoints, otherwise
    // it returns 'true'.
    public boolean isDead() {
        return hitpoints <= 0;
    }

    // Returns a representation of this player with his name and his hitpoints in
    // parentheses. If this fighter is dead, hitpoints are 0.
    public String toString() {
        return name + "(" + hitpoints + ")";
    }
}

// A team consists of multiple fighters stored in a ring list (all nodes have a successor node != null).
// One fighter of the team is active and engages a one-on-one fight with the active fighter
// of the opponent team. After that, the next fighter becomes active in each team.
// TODO: define further classes for the implementation of the ring list, if needed (either in this file
//  or in separate files)
//
class Team {
    private RingListElement el;
    private RingListElement active;

    public Team() {
        el = new RingListElement(null, null);
        el.setNext(el);
    }

    // Initializes this team with the fighters from 'fighters' and sets the first
    // fighter in the list active.
    // Preconditions: fighters != null and has no entries equal to 'null' (need not be checked).
    public Team(Fighter[] fighters) {
        this();
        for (Fighter f : fighters) {
            add(f);
        }
        active = el.getNext();
    }

    private void add(Fighter f) {
        el.setValue(f);
        el.setNext(el = new RingListElement(null, el.getNext()));
    }

    // Simulates a one-on-one fight: the active fighter of this team fights
    // the active fighter of team 'opponent' by calling the method 'fight' of class 'Fighter'.
    // Dead fighters are removed from their team (from the ring list) after
    // the one-on-one fight. After a fight, the next fighter becomes active in each team.
    // If 'this' or 'opponent' is empty, calling this method has no effect.
    // Precondition: opponent != null (needs not be checked).
    public void fight(Team opponent) {
        active.getValue().fights(opponent.active.getValue());
        if (active.getValue().isDead() && opponent.active.getValue().isDead()) {
            RingListElement te = active;
            while (te.getNext() != active)
                te = te.getNext();

            RingListElement oe = opponent.active;
            while (oe.getNext() != opponent.active)
                oe = oe.getNext();
            oe.setNext(oe.getNext().getNext());

            active = active.getNext() == el ? active.getNext().getNext() : active.getNext();
            te.setNext(te.getNext().getNext());
            opponent.active = opponent.active.getNext() == opponent.el ? opponent.active.getNext().getNext() : opponent.active.getNext();
            return;
        }
        if (active.getValue().isDead()) {
            RingListElement e = active;
            while (e.getNext() != active)
                e = e.getNext();
            active = active.getNext() == el ? active.getNext().getNext() : active.getNext();
            e.setNext(e.getNext().getNext());
            opponent.active = opponent.active.getNext() == opponent.el ? opponent.active.getNext().getNext() : opponent.active.getNext();
        } else {
            RingListElement e = opponent.active;
            while (e.getNext() != opponent.active)
                e = e.getNext();
            opponent.active = opponent.active.getNext() == opponent.el ? opponent.active.getNext().getNext() : opponent.active.getNext();
            e.setNext(e.getNext().getNext());
            active = active.getNext() == el ? active.getNext().getNext() : active.getNext();
        }
    }

    // Returns 'true' if there is no fighter in the team.
    public boolean isEmpty() {
        return el.getNext() == el;
    }

    // Returns a representation of this team with all its (remaining) fighters in
    // brackets in order of their insertion. The active fighter is marked by '*'.
    // Returns "[]" if the team is empty.
    public String toString() {
        return "[" + toStringR() + "]";
    }

    private String toStringR() {
        String res = "";
        RingListElement e = el.getNext();
        while (e != el) {
            if (e == active) {
                res += "*" + e.getValue().toString() + "*, ";
            } else
                res += e.getValue().toString() + ", ";
            e = e.getNext();
        }
        return res;
    }
}

class RingListElement {
    private Fighter value;
    private RingListElement next;

    public RingListElement(Fighter value, RingListElement next) {
        this.value = value;
        this.next = next;
    }

    public RingListElement(Fighter value) {
        this.value = value;
    }

    public Fighter getValue() {
        return value;
    }

    public void setValue(Fighter value) {
        this.value = value;
    }

    public RingListElement getNext() {
        return next;
    }

    public void setNext(RingListElement next) {
        this.next = next;
    }
}