package vowi.Test2_18S.expression;

import java.util.*;

/**
 * B3
 * Schreiben bzw. vervollständigen Sie bitte gekennzeichnete Programmteile wie in den Kommentaren beschrieben.
 * Verändern Sie aber nicht vorgegebene Methodenköpfe oder andere vorgegebene Programmteile.
 * Neben den Testfällen in 'main' müssen auch alle Zusicherungen in den Kommentaren und Bedingungen
 * auf dem Aufgabenblatt gelten.
 */

public class PraxisAufgabe {

    /*
    Aufgabe: siehe Angabeblatt

        Ergänzen Sie bitte fehlende Teile der Klassen
        an den mit TODO gekennzeichneten Stellen.
        Auch für teilweise korrekte Lösungen werden Punkte vergeben.

    */

    // 'main' enthält vorgefertigte Testfälle jeweils mit Sollausgabe als Kommentar.
    // Ergänzen weiterer Testfälle ist hier erlaubt.
    // Das Testen wird nicht beurteilt.
    public static void main(String[] args) {

        Expression a = new Sum(new Sum(new Number(5), new Number(-3)), new Number(7));
        System.out.println(a); // 5-3+7

        a = a.add(new Number(-10));
        System.out.println(a); // 5-3+7-10

        for (Integer i : a) {
            System.out.print(i + " ");
        }
        System.out.println();
        // 5 -3 7 -10

        Expression n = new Number(13);
        Iterator iter = n.iterator();
        System.out.println(iter.hasNext()); // true
        System.out.println(iter.next()); // 13

        try {
            System.out.println(iter.next());
        } catch (NoSuchElementException e) {
            System.out.println(e.getMessage()); // No more numbers!
        }

        a = a.add(a);
        System.out.println(a); // 5-3+7-10+5-3+7-10

        System.out.println(a.eval()); // -2
    }
}

// Iterierbare Objekte mit ganzzahligen Elementen
interface Iterable extends java.lang.Iterable<java.lang.Integer> {
    Iterator iterator();
}

// Ein Iterator über ganze Zahlen.
interface Iterator extends java.util.Iterator<java.lang.Integer> {
    boolean hasNext();

    java.lang.Integer next();
}

// Repräsentiert einen konstanten arithmetischen Summenausdruck
// (Ausdruck in dem Zahlen durch Addition verknüpft werden).
interface Expression extends Iterable {
    // Liefert den Wert des Ausdrucks.
    int eval();

    // Liefert eine lesbare Darstellung des Ausdrucks ohne Klammern.
    String toString();

    // Liefert einen neuen Ausdruck, in dem 'exp' als rechter Summand hinzugefügt wurde.
    Expression add(Expression exp);
}

// Diese Klasse repräsentiert eine ganze Zahl. Ihr Iterator liefert nur eine Zahl und ist danach verbraucht.
class Number implements Expression {
    private int num;

    public Number(int num) {
        this.num = num;
    }

    public int eval() {
        return num;
    }

    public Sum add(Expression exp) {
        return new Sum(exp, this);
    }

    public String toString() {

        return num >= 0 ? "+" + num : "" + num;
    }

    public Iterator iterator() {
        return new Iterator() {
            boolean hasNext = true;

            @Override
            public boolean hasNext() {
                return hasNext;
            }

            @Override
            public Integer next() {
                if (hasNext) {
                    hasNext = false;
                    return num;
                }
                throw new NoSuchElementException("No Element");
            }
        };
    }
}

// Eine Summe mit linkem und rechtem Summanden. Der Iterator iteriert über alle Zahlen des Ausdrucks von links nach rechts.
class Sum implements Expression {
    private Expression expressionLeft;
    private Expression expressionRight;

    // Konstruktor
    public Sum(Expression left, Expression right) {
        this.expressionLeft = left;
        this.expressionRight = right;
    }

    public Sum add(Expression exp) {
//        if (expressionRight == null) {
//            expressionRight = exp;
//            //return this //NEVER RETURN THIS!! it may generate to a loop if you call exp1 = exp1.add(exp1);
//            return new Sum(expressionLeft, exp);
//        }
//        return new Sum(expressionLeft, new Sum(expressionRight, exp));
        return new Sum(this, exp);
    }

    public int eval() {
        if (expressionRight != null)
            return expressionLeft.eval() + expressionRight.eval();
        else
            return expressionLeft.eval();
    }

    public Iterator iterator() {
        return new Iterator() {
            Iterator iterR = expressionRight.iterator();
            Iterator iterL = expressionLeft.iterator();

            @Override
            public boolean hasNext() {
                return iterL.hasNext() || iterR.hasNext();
            }

            @Override
            public Integer next() {
                if (iterL.hasNext()) {
                    return iterL.next();
                } else {
                    return iterR.next();
                }
            }
        };
    }

    public String toString() {
        String res = "";
        Iterator iter = iterator();

        for (int i : this) {
            i = iter.next();
            if (res.equals("") && i > 0)
                res += i;
            else
                res += i < 0 ? "" + i : "+" + i;
        }
        return res;
    }
}