package vowi.Test2_18S.interval;

import java.util.*;

/**
 * A1
 * Schreiben bzw. vervollständigen Sie bitte gekennzeichnete Programmteile wie in den Kommentaren beschrieben.
 * Verändern Sie aber nicht vorgegebene Methodenköpfe oder andere vorgegebene Programmteile.
 * Neben den Testfällen in 'main' müssen auch alle Zusicherungen in den Kommentaren und Bedingungen
 * auf dem Aufgabenblatt gelten.
 */

public class PraxisAufgabe {

    /*
    Aufgabe: siehe Angabeblatt

        Ergänzen Sie bitte fehlende Teile der Klassen
        an den mit TODO gekennzeichneten Stellen.
        Auch für teilweise korrekte Lösungen werden Punkte vergeben.

    */

    // 'main' enthält vorgefertigte Testfälle jeweils mit Sollausgabe als Kommentar.
    // Ergänzen weiterer Testfälle ist hier erlaubt.
    // Das Testen wird nicht beurteilt.
    public static void main(String[] args) {
        Interval i1 = new NonEmpty(-1, 10);
        Interval i2 = new NonEmpty(5, 20);
        Interval i3 = new NonEmpty(20, 25);

        try {
            i3 = new NonEmpty(20, -25);
        } catch (IllegalArgumentException e) {
            System.out.println(e.getMessage()); // lower (20) must not be greater than upper (-25)!
        }

        System.out.println(i1.intersect(i2)); //[5, 10]
        System.out.println(i2.intersect(i3)); //[20, 20]
        System.out.println(i1.intersect(i3)); //[]

        System.out.println(i1.containsAll(i1.intersect(i2))); // true

        Iterator iter = Empty.EMPTY.iterator();
        try {
            iter.next();
        } catch (NoSuchElementException e) {
            System.out.println(e.getMessage()); // EMPTY has no elements!
        }

        Interval i4 = new NonEmpty(-5, 20);

        System.out.println();
        System.out.println("My tests");
        System.out.println(i1.intersect(i4));//[-1, 10]
        System.out.println(i4.intersect(i1));//[-1, 10]
        System.out.println(i1);//[-1, 10]
        for (int i : i1) {
            System.out.print(i + " ");
        }
        //-1 0 1 2 3 4 5 6 7 8 9 10
        System.out.println();
        System.out.println(i1 + " " + i2);//[-1, 10] [5, 20]
        System.out.println(i1.containsAll(i2));//false
        System.out.println(i4.containsAll(i1));//true

        System.out.println(Empty.EMPTY.equals(Empty.EMPTY));//true
        System.out.println(Empty.EMPTY.hashCode());//hashcode

        Interval i5 = new NonEmpty(-5, 20);
        Interval i6 = new NonEmpty(-5, 20);

        System.out.println(i5.equals(Empty.EMPTY));//false
        System.out.println(i5.equals(i6));//true
        System.out.println(i5.hashCode());//hashcode

        Interval i7 = new NonEmpty(1, 1);
        Iterator iter2 = i7.iterator();
        try{
            iter2.next();
            iter2.next();
        }catch (NoSuchElementException e){
            System.out.println(e.getMessage());
        }
        //No more Elements!
    }
}

// Ein 'Interval' ist eine aufsteigend iterierbare Menge
// von aufeinander folgenden ganzen Zahlen und stellt einfache
// Mengenoperationen zur Verfügung.
interface Interval extends Iterable {

    // liefert 'true' genau dann, wenn 'n' in der Menge enthalten ist
    boolean contains(Integer n);

    // liefert 'true' genau dann, wenn alle Zahlen aus 'iv' enthalten sind.
    boolean containsAll(Interval iv);

    // liefert ein Objekt von 'Interval', das der Schnittmenge mit 'iv' entspricht.
    Interval intersect(Interval iv);

    // liefert 'true' genau dann, wenn das Objekt die leere Menge repräsentiert.
    boolean isEmpty();

    // liefert 'true' genau dann, wenn die Menge genau die gleichen Zahlen
    // enthält wie 'o' (und 'o' auch ein 'Interval'-Objekt ist).
    boolean equals(Object o);

    // liefert den hashcode
    int hashCode();
}

// Iterierbare Objekte mit ganzzahligen Elementen
interface Iterable extends java.lang.Iterable<Integer> {
    Iterator iterator();
}

// Ein Iterator über ganze Zahlen.
interface Iterator extends java.util.Iterator<Integer> {
    boolean hasNext();

    Integer next();
}

// 'Empty' repräsentiert die leere Menge.
class Empty implements Interval {

    // Öffentliches Objekt zur Repräsentation der leeren Menge.
    public static final Empty EMPTY = new Empty();

    // Konstruktor (wird nur von 'Empty' genutzt)
    private Empty() {
    }

    @Override
    public boolean contains(Integer n) {
        return false;
    }

    @Override
    public boolean containsAll(Interval iv) {
        return false;
    }

    @Override
    public Interval intersect(Interval iv) {
        return this;
    }

    @Override
    public boolean isEmpty() {
        return false;
    }

    @Override
    public Iterator iterator() {
        return new Iterator() {
            @Override
            public boolean hasNext() {
                return false;
            }

            @Override
            public Integer next() {
                throw new NoSuchElementException("EMPTY has no elements!");
            }
        };
    }

    @Override
    public String toString() {
        return "[]";
    }
}

// 'NonEmpty' repräsentiert eine aufsteigend iterierbare Menge von aufeinander folgenden ganzen Zahlen,
// die mindestens eine Zahl enthält. 'NonEmpty' wird durch Untergrenze 'lower' und Obergenze 'upper'
// des Intervalls bestimmt.
class NonEmpty implements Interval {

    private int lower;
    private int upper;

    public NonEmpty(int lower, int upper) {
        if(lower > upper)
            throw new IllegalArgumentException("lower ("+lower+") must not be greater than upper ("+upper+")!");
        this.lower = lower;
        this.upper = upper;
    }

    @Override
    public boolean contains(Integer n) {
        return lower <= n && n <= upper;
    }

    @Override
    public boolean containsAll(Interval iv) {
        if (iv == null || iv.getClass() != this.getClass())
            return false;
        NonEmpty ne = (NonEmpty) iv;
        return contains(ne.lower) && contains(ne.upper);
    }

    @Override
    public Interval intersect(Interval iv) {
        if (iv == null || iv.getClass() != this.getClass())
            return Empty.EMPTY;
        NonEmpty ne = (NonEmpty) iv;
        if (!contains(ne.lower) && !contains(ne.upper) && !ne.contains(lower) && !ne.contains(upper))
            return Empty.EMPTY;

        int l = contains(ne.lower) ? ne.lower : lower;
        int u = contains(ne.upper) ? ne.upper : upper;
        return new NonEmpty(l, u);
    }

    @Override
    public boolean isEmpty() {
        return lower == upper;
    }

    @Override
    public Iterator iterator() {
        return new Iterator() {
            int l = lower;

            @Override
            public boolean hasNext() {
                l = l >= lower ? l : lower;
                return l <= upper;
            }

            @Override
            public Integer next() {
                if (hasNext()) {
                    l = l >= lower ? l : lower;
                    return l++;
                }
                throw new NoSuchElementException("No more Elements!");
            }
        };
    }

    @Override
    public String toString() {
        return "[" + lower + ", " + upper + "]";
    }

    @Override
    public boolean equals(Object o) {
        if(o == null || o.getClass() != this.getClass())
            return false;
        if(o == this)
            return true;
        NonEmpty ne = (NonEmpty) o;
        return this.lower == ne.lower && this.upper == ne.upper;
    }

    @Override
    public int hashCode() {
        return Objects.hash(lower, upper);
    }
}