package com.company;

import java.util.List;

/**
 * Schreiben bzw. vervollständigen Sie bitte gekennzeichnete Programmteile wie in den Kommentaren beschrieben.
 * Verändern Sie aber nicht vorgegebene Methodenköpfe oder andere vorgegebene Programmteile.
 * Die erfolgreiche Ausführung der Testfälle in 'main' stellt nicht sicher, dass die Lösung korrekt ist.
 */

public class TestS18SlotC {

    /*
    Aufgabe: Klassen und Objekte, rekursive Datenstrukturen.

        Ergänzen Sie bitte fehlende Teile der Klassen 'Number_2' und 'BigNumber' entsprechend den Kommentaren
        an den mit TODO gekennzeichneten Stellen.

    Punkte (in Summe maximal 75):

        20 Punkte in Summe für die Klasse 'Number_2'.

        55 Punkte in Summe für die Klasse 'BigNumber', aufgeteilt wie folgt:

            40 Punkte für die Implementierung einer einfach oder doppelt verketteten Liste zum
            Speichern beliebig vieler (8er-Gruppen von) Stellen der Zahl.

            15 Punkte für die korrekte Funktionalität (unabhängig von der Implementierung).

            Hinweis: Wird statt einer eigenen Implementierung der verketteten Liste eine vorgefertigte
            Java Collection Klasse oder ein Array genutzt, können für die Klasse 'BigNumber'
            maximal 15 Punkte erreicht werden.

    Die Anzahl der dargestellten führenden Nullen ist beliebig,
    d.h. Lösungen mit und ohne führende Nullen sind erlaubt.

    Auch für teilweise korrekte Lösungen werden Punkte vergeben.

    */

    // 'main' ist zum Testen verwendbar und kann beliebig geändert werden.
    // Das Testen wird nicht beurteilt.
    public static void main(String[] args) {
        // Hinweis: Die Anzahl der dargestellten führenden Nullen darf von den hier gezeigten
        // Sollausgaben abweichen.
        Number n = new Number(99999154);
        System.out.println(n.toString()); // 99999154

        for (int i = 0; i < 1000000; i++) {
            n.increment();
        }
        System.out.println(n.toString()); // 00999154

        for (int i = 0; i < 1000000; i++) {
            n.increment();
        }
        System.out.println(n.toString()); // 01999154

        BigNumber bn = new BigNumber(Long.MAX_VALUE);
        System.out.println(bn.toString());  // 000009223372036854775807

        for (int i = 0; i < 1234567; i++) {
            bn.increment();
        }
        System.out.println(bn.toString()); // 000009223372036856010374

        bn = new BigNumber("725473526600001886534325642623");
        bn.increment();
        System.out.println(bn.toString()); // 00725473526600001886534325642624
        bn.setDigit(50, 2);
        System.out.println(bn.toString()); // 00000200000000000000000000725473526600001886534325642624
        bn.setDigit(50, 0);
        System.out.println(bn.toString()); // 00000000000000000000000000725473526600001886534325642624
    }
}

// Diese Klasse repräsentiert eine positive ganze Zahl mit maximal 8 Stellen
// im Dezimalsystem.
class Number {

    // Anzahl der Stellen
    public final static int nDigits = 8;
    private int[] number = new int[nDigits];


    // Initialisiert dieses Objekt mit dem Wert 'n'.
    // Es gilt: 0 <= n && n < Math.pow(10,nDigits).
    public Number(int n) {
        int index = nDigits - 1;
        while (n > 0) {
            number[index] = n % 10;
            index -= 1;
            n /= 10;
        }
    }

    public Number(long n) {
        int index = nDigits - 1;
        while (n > 0) {
            number[index] = (int)n % 10;
            index -= 1;
            n /= 10;
        }
    }

    // Initialisiert dieses Objekt mit dem gleichen Wert wie 'n'.
    public Number(Number n) {
        for (int i = 0; i < nDigits; i++) {
            number[i] = n.number[i];
        }

    }

    // Liefert eine lesbare Respräsentation dieses Objekts.
    public String toString() {
        String res = "";
        for (int i = 0; i < nDigits; i++) {
            res += number[i];
        }
        return res;
    }

    // Liefert die Ziffer an der Stelle pos.
    // Es gilt: 0 <= pos && pos < nDigits.
    public int getDigit(int pos) {
        int newPos = nDigits - pos - 1;
        return number[newPos];
    }

    // Ersetzt die Ziffer an der Stelle 'pos' durch die Ziffer 'd'.
    // Es gilt: 0 <= pos && pos < nDigits und 0 <= d && d <= 9.
    public void setDigit(int pos, int d) {
        int newPos = nDigits - pos - 1;
        number[newPos] = d;
    }

    // Inkrementiert die durch dieses Objekt dargestellte Zahl. Wird dabei der Wertebereich überschritten,
    // das heißt, die Zahl bekommt dadurch mehr als 8 Stellen, werden vom Ergebnis nur die ersten 8 (niederwertigsten)
    // Stellen gespeichert und durch den zurückgegebenen Wert 'true' wird signalisiert, dass es einen Überlauf
    // gegeben hat. Ohne Überlauf wird 'false' zurückgegeben.
    public boolean increment() {
        boolean ueberlauf = false;
        for (int i = nDigits - 1; i >= 0; i--) {
            if (i == 0 && number[i] == 9)
                ueberlauf = true;
            if (number[i] != 9)
                number[i] = number[i] + 1;
            else
                number[i] = 0;

            if (number[i] != 0)
                break;
        }
        return ueberlauf;
    }
}


// Die Klasse 'BigNumber' repräsentiert eine positive ganze Zahl mit beliebigem Betrag. Der Wertebereich beginnt
// bei 0 und kann sowohl Integer.MAX_VALUE als auch Long.MAX_VALUE überschreiten, ohne dass die Funktionalität
// beeinträchtigt wird. Dies wird dadurch erreicht, dass BigNumber intern eine verkette Liste von 'Number'-Objekten
// verwendet. z.B. wird 750000456123456789 dargestellt durch drei Knoten 23456789 -> 4561 -> 75 -> null.
class BigNumber {
    ListElement first;
    ListElement last;

    // Konstruktor: Initialisiert dieses Objekt mit 'n'.
    public BigNumber(long n) {
        long i = n % 100000000;
        first = last = new ListElement(null, new Number(i));
        n /= 100000000;
        ListElement prev = first;
        do {
            i = n % 100000000;
            ListElement el = new ListElement(prev, new Number(i));
            prev.setNext(el);
            prev = prev.getNext();
            n /= 100000000;
        } while (n > 0);

        while (last.getNext() != null) {
            last = last.getNext();
        }
    }

    // Inkrementiert die Zahl.
    public void increment() {
        ListElement el = first;
        while (el.getValue().increment()) {
            if (el.getPrevious() == null) {
                last.setNext(new ListElement(null, new Number(1)));
                last = last.getNext();
            }
            el = el.getNext();
        }
    }

    // Ersetzt die Ziffer an der Stelle 'pos' durch die Ziffer 'd'.
    // Es gilt: pos >= 0 und 0 <= d && d <= 9.
    public void setDigit(int pos, int d) {
        ListElement el = first;

        while (pos > 8) {
            pos -= 8;
            if(el.getNext() == null){
                last.setNext(new ListElement(last, new Number(0)));
                last = last.getNext();
            }
            el = el.getNext();
        }
        el.getValue().setDigit(pos, d);
    }

    // Konstruktor: Initialisiert dieses Objekt mit der Zahl, die als String angegeben wird.
    // 's' sei ein String, der ausschließlich Ziffern enthält.
    public BigNumber(String s) {
        int length = s.length() <= 8 ? s.length() : 8;
        long i = Long.valueOf(s.substring(s.length() - length));
        first = last = new ListElement(null, new Number(i));
        s = s.substring(0, s.length() - 8);
        ListElement prev = first;
        do {
            length = s.length() <= 8 ? s.length() : 8;
            i = Long.valueOf(s.substring(s.length() - length));
            ListElement el = new ListElement(prev, new Number(i));
            prev.setNext(el);
            prev = prev.getNext();
            s = s.substring(0, s.length() - length);
        } while (s.length() > 0);

        while (last.getNext() != null) {
            last = last.getNext();
        }

    }

    // Liefert eine lesbare Repräsentation des Objekts (siehe Beispiele in 'main').
    public String toString() {
        ListElement el = last;
        String res = "";
        do {
            res += el.getValue().toString();
            el = el.getPrevious();
        } while (el != null);
        return res;
    }
}

class ListElement {
    ListElement next, previous;
    Number value;

    public ListElement(ListElement previous, Number value) {
        this.previous = previous;
        this.value = value;
    }

    public ListElement getNext() {
        return next;
    }

    public void setNext(ListElement next) {
        this.next = next;
    }

    public ListElement getPrevious() {
        return previous;
    }

    public void setPrevious(ListElement previous) {
        this.previous = previous;
    }

    public Number getValue() {
        return value;
    }

    public void setValue(Number value) {
        this.value = value;
    }
}
